///////////////////////////////////////////////////////////////////////////////
//
//  This file describes the interface of the SETL-like sublanguage compiler.
//
///////////////////////////////////////////////////////////////////////////////
#ifndef SETL_data_mapping_compiler_h
#define SETL_data_mapping_compiler_h

#include "matchcom.h"
#include "setl-ast.h"
#include "env.h"

///////////////////////////////////////////////////////////////////////////////
//
//  Forward declarations
//
///////////////////////////////////////////////////////////////////////////////
datatype Time and Space;

///////////////////////////////////////////////////////////////////////////////
// 
//  Type checking methods
//
///////////////////////////////////////////////////////////////////////////////
extern Ty         type_of (Exp,  const Env&);  // type checking an expression
extern Tys        type_of (Exps, const Env&);  // type checking an expression list
extern .[Id,Ty]   type_of (LabExp,  const Env&);  // type checking an labeled expression
extern .[Ids,Tys] type_of (LabExps, const Env&);  // type checking an labeled expression list
extern Env  type_of (Def,   const Env&);  // type checking a definition
extern Env  type_of (Defs,  const Env&);  // type checking a definition list
extern void type_of (Stmt,  const Env&);  // type checking a statement 
extern void type_of (Stmts, const Env&);  // type checking a statement list

///////////////////////////////////////////////////////////////////////////////
//
//  We inherit from the code generator and the match compiler.
//
///////////////////////////////////////////////////////////////////////////////
class SETLCompiler : virtual public CodeGen,
                     virtual public MatchCompiler 
{   
   SETLCompiler(const SETLCompiler&);       // no copy constructor
   void operator = (const SETLCompiler&);   // no assignment
public:
   ////////////////////////////////////////////////////////////////////////////
   // 
   //  Constructor and destructor
   //
   ////////////////////////////////////////////////////////////////////////////
            SETLCompiler();
   virtual ~SETLCompiler();

   ////////////////////////////////////////////////////////////////////////////
   //
   //  Generate SETL-like statements and definitions
   //
   ////////////////////////////////////////////////////////////////////////////
   void gen_setl (Stmt);
   void gen_setl (Def);

private:

   ////////////////////////////////////////////////////////////////////////////
   // 
   //  Domain inference methods
   //
   ////////////////////////////////////////////////////////////////////////////

   ////////////////////////////////////////////////////////////////////////////
   // 
   //  Time and space complexity inference methods
   //
   ////////////////////////////////////////////////////////////////////////////
   .[Time,Space] complexity_of (Exp, const Env&);
   .[Time,Space] complexity_of (Stmt, const Env&);

   ////////////////////////////////////////////////////////////////////////////
   // 
   //  Code generation methods
   //
   ////////////////////////////////////////////////////////////////////////////
   void gen_def   (Def);      // code generation for definitions
   void gen_defs  (Defs);     // code generation for definition lists
   void gen_stmt  (Stmt);     // code generation for statements
   void gen_stmts (Stmts);    // code generation for statement lists
   void gen_exp   (Exp);      // code generation for expressions
   void gen_exps  (Exps);     // code generation for expression lists
};

#endif
